@@ -10,16 +10,15 @@
 #include "chafafunc.h"
 #include "directorytree.h"
 #include "playlist.h"
+#include "playlist_ui.h"
+#include "search_ui.h"
 #include "songloader.h"
 #include "sound.h"
 #include "term.h"
 #include "utils.h"
 #include "visuals.h"
+#include "common_ui.h"
 
-extern int mainColor;
-extern int artistColor;
-extern int enqueuedColor;
-extern int titleColor;
 extern const char VERSION[];
 extern bool coverEnabled;
 extern bool uiEnabled;
@@ -44,6 +43,7 @@ extern double totalPauseSeconds;
 extern double seekAccumulatedSeconds;
 extern bool allowChooseSongs;
 extern int chosenLibRow;
+extern int chosenSearchResultRow;
 extern int chosenRow;
 extern int chosenNodeId;
 extern bool useProfileColors;
@@ -62,6 +62,8 @@ void flipPrevPage();
 
 void showHelp(void);
 
+void setChosenDir(FileSystemEntry *entry);
+
 int printAbout(SongData *songdata);
 
 FileSystemEntry *getCurrentLibEntry();
@@ -82,6 +84,8 @@ void toggleShowLibrary();
 
 void toggleShowPlaylist(void);
 
+void toggleShowSearch(void);
+
 void showTrack();
 
 void setTextColorRGB2(int r, int g, int b);
@@ -93,4 +97,3 @@ char *getLibraryFilePath();
 void resetChosenDir();
 
 #endif
-
