@@ -54,6 +54,7 @@ extern bool skipOutOfOrder;
 extern Node *tryNextSong;
 extern struct timespec lastInputTime;
 extern struct timespec lastPlaylistChangeTime;
+extern bool skipFromStopped;
 
 extern UserData userData;
 
@@ -65,7 +66,7 @@ Node *getNextSong();
 
 void handleRemove();
 
-void enqueueSongs();
+void enqueueSongs(FileSystemEntry *entry);
 
 void updateLastSongSwitchTime(void);
 
