@@ -580,6 +580,35 @@ int makePlaylist(int argc, char *argv[], bool exactSearch, const char *path)
         return 0;
 }
 
+// Function to generate the filename with the .m3u extension
+void generateM3UFilename(const char *basePath, const char *filePath, char *m3uFilename, size_t size) {
+    // Find the last occurrence of '/' in the file path to get the base name
+    const char *baseName = strrchr(filePath, '/');
+    if (baseName == NULL) {
+        baseName = filePath; // No '/' found, use the entire filename
+    } else {
+        baseName++; // Skip the '/' character
+    }
+
+    // Find the last occurrence of '.' in the base name
+    const char *dot = strrchr(baseName, '.');
+    if (dot == NULL) {
+        // No '.' found, copy the base name and append ".m3u"
+        if (basePath[strlen(basePath) - 1] == '/') {
+            snprintf(m3uFilename, size, "%s%s.m3u", basePath, baseName);
+        } else {
+            snprintf(m3uFilename, size, "%s/%s.m3u", basePath, baseName);
+        }
+    } else {
+        // Copy the base name up to the dot and append ".m3u"
+        size_t baseNameLen = dot - baseName;
+        if (basePath[strlen(basePath) - 1] == '/') {
+            snprintf(m3uFilename, size, "%s%.*s.m3u", basePath, (int)baseNameLen, baseName);
+        } else {
+            snprintf(m3uFilename, size, "%s/%.*s.m3u", basePath, (int)baseNameLen, baseName);
+        }
+    }
+}
 void writeM3UFile(const char *filename, PlayList *playlist)
 {
         FILE *file = fopen(filename, "w");
@@ -648,22 +677,15 @@ void savePlaylist(const char *path)
         {
                 return;
         }
-        
-        char playlistPath[MAXPATHLEN];
-        playlistPath[0] = '\0';
-        c_strcpy(playlistPath, sizeof(playlistPath), path);
 
-        if (playlistPath[0] == '\0')
-        {
+        if (playlist.head == NULL || playlist.head->song.filePath == NULL)
                 return;
-        }
+        
+        char m3uFilename[MAXPATHLEN];
 
-        if (playlistPath[strlen(playlistPath) - 1] != '/')
-                strcat(playlistPath, "/");
+        generateM3UFilename(path, playlist.head->song.filePath, m3uFilename, sizeof(m3uFilename));
 
-        strcat(playlistPath, playlistName);
-
-        writeM3UFile(playlistPath, &playlist);
+        writeM3UFile(m3uFilename, &playlist);
 }
 
 Node *deepCopyNode(Node *originalNode)
@@ -817,3 +839,128 @@ void createPlayListFromFileSystemEntry(FileSystemEntry *root, PlayList *list, in
 {
         traverseFileSystemEntry(root, list, playlistMax);
 }
+
+int isMusicFile(const char *filename)
+{
+    const char *extensions[] = {".m4a", ".aac",".mp4", ".mp3", ".ogg", ".flac",".wav", ".opus"};
+
+    size_t numExtensions = sizeof(extensions) / sizeof(extensions[0]);
+
+    for (size_t i = 0; i < numExtensions; i++)
+    {
+        if (strstr(filename, extensions[i]) != NULL)
+        {
+            return 1;
+        }
+    }
+
+    return 0;
+}
+
+int containsMusicFiles(FileSystemEntry *entry)
+{
+    if (entry == NULL)
+        return 0;
+
+    FileSystemEntry *child = entry->children;
+
+    while (child != NULL)
+    {
+        if (!child->isDirectory && isMusicFile(child->name))
+        {
+            return 1;
+        }
+        child = child->next;
+    }
+
+    return 0;
+}
+
+void addAlbumToPlayList(PlayList *list, FileSystemEntry *album, int playlistMax)
+{
+    FileSystemEntry *entry = album->children;
+
+    while (entry != NULL && list->count < playlistMax)
+    {
+        if (!entry->isDirectory && isMusicFile(entry->name))
+        {
+            addSongToPlayList(list, entry->fullPath, playlistMax);
+        }
+        entry = entry->next;
+    }
+}
+
+void addAlbumsToPlayList(FileSystemEntry *entry, PlayList *list, int playlistMax)
+{
+    if (entry == NULL || list->count >= playlistMax)
+        return;
+
+    if (entry->isDirectory && containsMusicFiles(entry))
+    {
+        addAlbumToPlayList(list, entry, playlistMax);
+    }
+
+    if (entry->isDirectory && entry->children != NULL)
+    {
+        addAlbumsToPlayList(entry->children, list, playlistMax);
+    }
+
+    if (entry->next != NULL)
+    {
+        addAlbumsToPlayList(entry->next, list, playlistMax);
+    }
+}
+
+void shuffleEntries(FileSystemEntry **array, size_t n)
+{
+    if (n > 1)
+    {
+        size_t i;
+        for (i = 0; i < n - 1; i++)
+        {
+            size_t j = i + rand() / (RAND_MAX / (n - i) + 1);
+            FileSystemEntry *temp = array[i];
+            array[i] = array[j];
+            array[j] = temp;
+        }
+    }
+}
+
+void collectAlbums(FileSystemEntry *entry, FileSystemEntry **albums, size_t *count)
+{
+    if (entry == NULL)
+        return;
+
+    if (entry->isDirectory && containsMusicFiles(entry))
+    {
+        albums[*count] = entry;
+        (*count)++;
+    }
+
+    if (entry->isDirectory && entry->children != NULL)
+    {
+        collectAlbums(entry->children, albums, count);
+    }
+
+    if (entry->next != NULL)
+    {
+        collectAlbums(entry->next, albums, count);
+    }
+}
+
+void addShuffledAlbumsToPlayList(FileSystemEntry *root, PlayList *list, int playlistMax)
+{
+    size_t maxAlbums = 2000; 
+    FileSystemEntry *albums[maxAlbums];
+    size_t albumCount = 0;
+
+    collectAlbums(root, albums, &albumCount);
+
+    srand(time(NULL));
+    shuffleEntries(albums, albumCount);
+
+    for (size_t i = 0; i < albumCount && list->count < playlistMax; i++)
+    {
+        addAlbumToPlayList(list, albums[i], playlistMax);
+    }
+}
