@@ -1,7 +1,5 @@
-#include <stdbool.h>
-#include <string.h>
 #include "settings.h"
-#include "stringfunc.h"
+
 /*
 
 settings.c
@@ -13,23 +11,32 @@ settings.c
 #ifndef MAXPATHLEN
 #define MAXPATHLEN 4096
 #endif
-#ifndef PATH_MAX
-#define PATH_MAX 4096
-#endif
 
-AppSettings settings;
+const char SETTINGS_FILE[] = "kewrc";
 
-const char OLD_SETTINGS_FILE[] = ".cue.conf";
-const char NEW_SETTINGS_FILE[] = "kewrc";
+void freeKeyValuePairs(KeyValuePair *pairs, int count)
+{
+        for (int i = 0; i < count; i++)
+        {
+                free(pairs[i].key);
+                free(pairs[i].value);
+        }
+
+        free(pairs);
+}
 
 AppSettings constructAppSettings(KeyValuePair *pairs, int count)
 {
         AppSettings settings;
         memset(&settings, 0, sizeof(settings));
         strncpy(settings.coverEnabled, "1", sizeof(settings.coverEnabled));
+        strncpy(settings.allowNotifications, "1", sizeof(settings.allowNotifications));
         strncpy(settings.coverAnsi, "0", sizeof(settings.coverAnsi));
         strncpy(settings.visualizerEnabled, "1", sizeof(settings.visualizerEnabled));
         strncpy(settings.useProfileColors, "1", sizeof(settings.useProfileColors));
+        strncpy(settings.hideLogo, "0", sizeof(settings.hideLogo));
+        strncpy(settings.hideHelp, "0", sizeof(settings.hideHelp));
+        strncpy(settings.cacheLibrary, "-1", sizeof(settings.cacheLibrary));
 
         strncpy(settings.volumeUp, "+", sizeof(settings.volumeUp));
         strncpy(settings.volumeUpAlt, "=", sizeof(settings.volumeUpAlt));
@@ -48,6 +55,7 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         strncpy(settings.seekBackward, "a", sizeof(settings.seekBackward));
         strncpy(settings.seekForward, "d", sizeof(settings.seekForward));
         strncpy(settings.savePlaylist, "x", sizeof(settings.savePlaylist));
+        strncpy(settings.updateLibrary, "u", sizeof(settings.updateLibrary));
         strncpy(settings.addToMainPlaylist, ".", sizeof(settings.addToMainPlaylist));
         strncpy(settings.hardPlayPause, " ", sizeof(settings.hardPlayPause));
         strncpy(settings.hardSwitchNumberedSong, "\n", sizeof(settings.hardSwitchNumberedSong));
@@ -57,17 +65,25 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         strncpy(settings.hardScrollDown, "[B", sizeof(settings.hardScrollDown));
         strncpy(settings.hardShowInfo, "OQ", sizeof(settings.hardShowInfo));
         strncpy(settings.hardShowInfoAlt, "[[B", sizeof(settings.hardShowInfoAlt));
-        strncpy(settings.hardShowKeys, "[15~", sizeof(settings.hardShowKeys));
-        strncpy(settings.hardShowKeysAlt, "[[E", sizeof(settings.hardShowKeysAlt));
+        strncpy(settings.hardShowKeys, "[17~", sizeof(settings.hardShowKeys));
+        strncpy(settings.hardShowKeysAlt, "[17~", sizeof(settings.hardShowKeysAlt));
         strncpy(settings.hardShowTrack, "OS", sizeof(settings.hardShowTrack));
         strncpy(settings.hardShowTrackAlt, "[[D", sizeof(settings.hardShowTrackAlt));
         strncpy(settings.hardEndOfPlaylist, "G", sizeof(settings.hardEndOfPlaylist));
         strncpy(settings.hardShowLibrary, "OR", sizeof(settings.hardShowLibrary));
         strncpy(settings.hardShowLibraryAlt, "[[C", sizeof(settings.hardShowLibraryAlt));
+        strncpy(settings.hardShowSearch, "[15~", sizeof(settings.hardShowSearch));
+        strncpy(settings.hardShowSearchAlt, "[[E", sizeof(settings.hardShowSearchAlt));
         strncpy(settings.hardNextPage, "[6~", sizeof(settings.hardNextPage));
         strncpy(settings.hardPrevPage, "[5~", sizeof(settings.hardPrevPage));
         strncpy(settings.hardRemove, "[3~", sizeof(settings.hardRemove));
+        strncpy(settings.lastVolume, "100", sizeof(settings.lastVolume));
+        strncpy(settings.color, "6", sizeof(settings.color));
+        strncpy(settings.artistColor, "6", sizeof(settings.artistColor));
+        strncpy(settings.titleColor, "6", sizeof(settings.titleColor));
+        strncpy(settings.enqueuedColor, "7", sizeof(settings.enqueuedColor));
         strncpy(settings.quit, "q", sizeof(settings.quit));
+        strncpy(settings.hardQuit, "\x1B", sizeof(settings.hardQuit));
 
         if (pairs == NULL)
         {
@@ -178,10 +194,50 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.quit, sizeof(settings.quit), "%s", pair->value);
                 }
+                else if (strcmp(stringToLower(pair->key), "lastvolume") == 0)
+                {
+                        snprintf(settings.lastVolume, sizeof(settings.lastVolume), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "allownotifications") == 0)
+                {
+                        snprintf(settings.allowNotifications, sizeof(settings.allowNotifications), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "color") == 0)
+                {
+                        snprintf(settings.color, sizeof(settings.color), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "artistcolor") == 0)
+                {
+                        snprintf(settings.artistColor, sizeof(settings.artistColor), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "enqueuedcolor") == 0)
+                {
+                        snprintf(settings.enqueuedColor, sizeof(settings.enqueuedColor), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "titlecolor") == 0)
+                {
+                        snprintf(settings.titleColor, sizeof(settings.titleColor), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "hidelogo") == 0)
+                {
+                        snprintf(settings.hideLogo, sizeof(settings.hideLogo), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "hidehelp") == 0)
+                {
+                        snprintf(settings.hideHelp, sizeof(settings.hideHelp), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "cachelibrary") == 0)
+                {
+                        snprintf(settings.cacheLibrary, sizeof(settings.cacheLibrary), "%s", pair->value);
+                }
                 else if (strcmp(stringToLower(pair->key), "quit") == 0)
                 {
                         snprintf(settings.quit, sizeof(settings.quit), "%s", pair->value);
                 }
+                else if (strcmp(stringToLower(pair->key), "updatelibrary") == 0)
+                {
+                        snprintf(settings.updateLibrary, sizeof(settings.updateLibrary), "%s", pair->value);
+                }
         }
 
         freeKeyValuePairs(pairs, count);
@@ -228,79 +284,12 @@ KeyValuePair *readKeyValuePairs(const char *file_path, int *count)
         return pairs;
 }
 
-void freeKeyValuePairs(KeyValuePair *pairs, int count)
-{
-        for (int i = 0; i < count; i++)
-        {
-                free(pairs[i].key);
-                free(pairs[i].value);
-        }
-
-        free(pairs);
-}
-
-const char *getHomePath()
-{
-        const char *xdgHome = getenv("XDG_HOME");
-        if (xdgHome == NULL)
-        {
-                xdgHome = getenv("HOME");
-                if (xdgHome == NULL)
-                {
-                        struct passwd *pw = getpwuid(getuid());
-                        if (pw != NULL)
-                        {
-                                char *home = (char *)malloc(PATH_MAX);
-                                strcpy(home, pw->pw_dir);
-                                return home;
-                        }
-                }
-        }
-        return xdgHome;
-}
-
-const char *getConfigPath()
-{
-        const char *xdgConfig = getenv("XDG_CONFIG_HOME");
-        if (xdgConfig == NULL)
-        {
-                // If XDG_CONFIG_HOME is not set, fall back to the default location
-                const char *home = getHomePath();
-                if (home != NULL)
-                {
-                        // Construct the default configuration path
-                        char *configPath = (char *)malloc(PATH_MAX);
-                        if (configPath != NULL)
-                        {
-                                snprintf(configPath, PATH_MAX, "%s/.config", home);
-                                if (isDirectory(configPath))
-                                {
-                                        xdgConfig = configPath;
-                                }
-                                else
-                                {
-                                        free(configPath); // Free the allocated memory
-                                        return home;
-                                }
-                        }
-                }
-                else
-                {
-                        struct passwd *pw = getpwuid(getuid());
-                        if (pw != NULL)
-                        {
-                                return pw->pw_dir;
-                        }
-                }
-        }
-        return xdgConfig;
-}
 const char *getDefaultMusicFolder()
 {
         const char *home = getHomePath();
         if (home != NULL)
         {
-                static char musicPath[PATH_MAX];
+                static char musicPath[MAXPATHLEN];
                 snprintf(musicPath, sizeof(musicPath), "%s/Music", home);
                 return musicPath;
         }
@@ -323,138 +312,168 @@ int getMusicLibraryPath(char *path)
         return 0;
 }
 
-void getConfigOld()
+void getConfig(AppSettings *settings)
 {
         int pair_count;
-        struct passwd *pw = getpwuid(getuid());
-        const char *homedir = pw->pw_dir;
+        char *configdir = getConfigPath();
         char *filepath = NULL;
 
-        size_t filepath_length = strlen(homedir) + strlen("/") + strlen(OLD_SETTINGS_FILE) + 1;
-        filepath = (char *)malloc(filepath_length);
-        strcpy(filepath, homedir);
-        strcat(filepath, "/");
-        strcat(filepath, OLD_SETTINGS_FILE);
-
-        KeyValuePair *pairs = readKeyValuePairs(filepath, &pair_count);
-
-        free(filepath);
-        settings = constructAppSettings(pairs, pair_count);
-
-        coverEnabled = (settings.coverEnabled[0] == '1');
-        coverAnsi = (settings.coverAnsi[0] == '1');
-        visualizerEnabled = (settings.visualizerEnabled[0] == '1');
-        useProfileColors = (settings.useProfileColors[0] == '1');
-        int temp = atoi(settings.visualizerHeight);
-        if (temp > 0)
-                visualizerHeight = temp;
-        getMusicLibraryPath(settings.path);
-}
-
-void getConfig()
-{
-        int pair_count;
-        const char *configdir = getConfigPath();
-        char *filepath = NULL;
-
-        size_t filepath_length = strlen(configdir) + strlen("/") + strlen(NEW_SETTINGS_FILE) + 1;
+        size_t filepath_length = strlen(configdir) + strlen("/") + strlen(SETTINGS_FILE) + 1;
         filepath = (char *)malloc(filepath_length);
         strcpy(filepath, configdir);
         strcat(filepath, "/");
-        strcat(filepath, NEW_SETTINGS_FILE);
-
-        if (existsFile(filepath) < 0)
-        {
-                free(filepath);
-                getConfigOld(); // Get config from the old location
-                return;
-        }
+        strcat(filepath, SETTINGS_FILE);
 
         KeyValuePair *pairs = readKeyValuePairs(filepath, &pair_count);
 
         free(filepath);
-        settings = constructAppSettings(pairs, pair_count);
+        *settings = constructAppSettings(pairs, pair_count);
 
-        coverEnabled = (settings.coverEnabled[0] == '1');
-        coverAnsi = (settings.coverAnsi[0] == '1');
-        visualizerEnabled = (settings.visualizerEnabled[0] == '1');
-        useProfileColors = (settings.useProfileColors[0] == '1');
-        int temp = atoi(settings.visualizerHeight);
-        if (temp > 0)
-                visualizerHeight = temp;
-        getMusicLibraryPath(settings.path);
+        allowNotifications = (settings->allowNotifications[0] == '1');
+        coverEnabled = (settings->coverEnabled[0] == '1');
+        coverAnsi = (settings->coverAnsi[0] == '1');
+        visualizerEnabled = (settings->visualizerEnabled[0] == '1');
+        useProfileColors = (settings->useProfileColors[0] == '1');
+        hideLogo = (settings->hideLogo[0] == '1');
+        hideHelp = (settings->hideHelp[0] == '1');
+        
+        int temp = atoi(settings->color);
+        if (temp >= 0)
+                mainColor = temp;
+        temp = atoi(settings->artistColor);
+        if (temp >= 0)
+                artistColor = temp;
+
+        temp = atoi(settings->enqueuedColor);
+        if (temp >= 0)
+                enqueuedColor = temp;
+
+        temp = atoi(settings->titleColor);
+        if (temp >= 0)
+                titleColor = temp;
+
+        int temp2 = atoi(settings->visualizerHeight);
+        if (temp2 > 0)
+                visualizerHeight = temp2;
+
+        int temp3 = atoi(settings->lastVolume);
+        if (temp3 >= 0)
+                setVolume(temp3);
+
+        int temp4 = atoi(settings->cacheLibrary);
+        if (temp4 >= 0)
+                cacheLibrary = temp4;
+
+        getMusicLibraryPath(settings->path);
+        free(configdir);
 }
 
-void setConfig()
+void setConfig(AppSettings *settings)
 {
         // Create the file path
-        const char *configdir = getConfigPath();
+        char *configdir = getConfigPath();
 
-        char *filepath = (char *)malloc(strlen(configdir) + strlen("/") + strlen(NEW_SETTINGS_FILE) + 1);
+        char *filepath = (char *)malloc(strlen(configdir) + strlen("/") + strlen(SETTINGS_FILE) + 1);
         strcpy(filepath, configdir);
         strcat(filepath, "/");
-        strcat(filepath, NEW_SETTINGS_FILE);
+        strcat(filepath, SETTINGS_FILE);
 
         FILE *file = fopen(filepath, "w");
         if (file == NULL)
         {
                 fprintf(stderr, "Error opening file: %s\n", filepath);
                 free(filepath);
+                free(configdir);
                 return;
         }
 
-        if (settings.coverEnabled[0] == '\0')
-                coverEnabled ? c_strcpy(settings.coverEnabled, sizeof(settings.coverEnabled), "1") : c_strcpy(settings.coverEnabled, sizeof(settings.coverEnabled), "0");
-        if (settings.coverAnsi[0] == '\0')
-                coverAnsi ? c_strcpy(settings.coverAnsi, sizeof(settings.coverAnsi), "1") : c_strcpy(settings.coverAnsi, sizeof(settings.coverAnsi), "0");
-        if (settings.visualizerEnabled[0] == '\0')
-                visualizerEnabled ? c_strcpy(settings.visualizerEnabled, sizeof(settings.visualizerEnabled), "1") : c_strcpy(settings.visualizerEnabled, sizeof(settings.visualizerEnabled), "0");
-        if (settings.useProfileColors[0] == '\0')
-                useProfileColors ? c_strcpy(settings.useProfileColors, sizeof(settings.useProfileColors), "1") : c_strcpy(settings.useProfileColors, sizeof(settings.useProfileColors), "0");
-        if (settings.visualizerHeight[0] == '\0')
+        if (settings->allowNotifications[0] == '\0')
+                allowNotifications ? c_strcpy(settings->allowNotifications, sizeof(settings->allowNotifications), "1") : c_strcpy(settings->allowNotifications, sizeof(settings->allowNotifications), "0");
+        if (settings->coverEnabled[0] == '\0')
+                coverEnabled ? c_strcpy(settings->coverEnabled, sizeof(settings->coverEnabled), "1") : c_strcpy(settings->coverEnabled, sizeof(settings->coverEnabled), "0");
+        if (settings->coverAnsi[0] == '\0')
+                coverAnsi ? c_strcpy(settings->coverAnsi, sizeof(settings->coverAnsi), "1") : c_strcpy(settings->coverAnsi, sizeof(settings->coverAnsi), "0");
+        if (settings->visualizerEnabled[0] == '\0')
+                visualizerEnabled ? c_strcpy(settings->visualizerEnabled, sizeof(settings->visualizerEnabled), "1") : c_strcpy(settings->visualizerEnabled, sizeof(settings->visualizerEnabled), "0");
+        if (settings->useProfileColors[0] == '\0')
+                useProfileColors ? c_strcpy(settings->useProfileColors, sizeof(settings->useProfileColors), "1") : c_strcpy(settings->useProfileColors, sizeof(settings->useProfileColors), "0");
+        if (settings->visualizerHeight[0] == '\0')
         {
-                sprintf(settings.visualizerHeight, "%d", visualizerHeight);
+                sprintf(settings->visualizerHeight, "%d", visualizerHeight);
         }
+        if (settings->hideLogo[0] == '\0')
+                hideLogo ? c_strcpy(settings->hideLogo, sizeof(settings->hideLogo), "1") : c_strcpy(settings->hideLogo, sizeof(settings->hideLogo), "0");
+        if (settings->hideHelp[0] == '\0')
+                hideHelp ? c_strcpy(settings->hideHelp, sizeof(settings->hideHelp), "1") : c_strcpy(settings->hideHelp, sizeof(settings->hideHelp), "0");
+
+        sprintf(settings->cacheLibrary, "%d", cacheLibrary);
+        sprintf(settings->lastVolume, "%d", getCurrentVolume());
 
         // Null-terminate the character arrays
-        settings.path[MAXPATHLEN - 1] = '\0';
-        settings.coverEnabled[1] = '\0';
-        settings.coverAnsi[1] = '\0';
-        settings.visualizerEnabled[1] = '\0';
-        settings.visualizerHeight[5] = '\0';
-        settings.useProfileColors[1] = '\0';
+        settings->path[MAXPATHLEN - 1] = '\0';
+        settings->coverEnabled[1] = '\0';
+        settings->coverAnsi[1] = '\0';
+        settings->visualizerEnabled[1] = '\0';
+        settings->visualizerHeight[5] = '\0';
+        settings->lastVolume[5] = '\0';
+        settings->useProfileColors[1] = '\0';
+        settings->allowNotifications[1] = '\0';
+        settings->hideLogo[1] = '\0';
+        settings->hideHelp[1] = '\0';
+        settings->cacheLibrary[5] = '\0';
 
         // Write the settings to the file
-        fprintf(file, "# Make sure that Kew is closed before editing this file in order for changes to take effect.\n\n");
+        fprintf(file, "# Make sure that kew is closed before editing this file in order for changes to take effect.\n\n");
 
-        fprintf(file, "path=%s\n", settings.path);
-        // fprintf(file, "useThemeColors=%s\n", settings.useThemeColors);
-        fprintf(file, "coverEnabled=%s\n", settings.coverEnabled);
-        fprintf(file, "coverAnsi=%s\n", settings.coverAnsi);
-        fprintf(file, "visualizerEnabled=%s\n", settings.visualizerEnabled);
-        fprintf(file, "visualizerHeight=%s\n", settings.visualizerHeight);
-        fprintf(file, "useProfileColors=%s\n\n", settings.useProfileColors);
-        fprintf(file, "# Key Bindings:\n\n");
-        fprintf(file, "volumeUp=%s\n", settings.volumeUp);
-        fprintf(file, "volumeUpAlt=%s\n", settings.volumeUpAlt);
-        fprintf(file, "volumeDown=%s\n", settings.volumeDown);
-        fprintf(file, "previousTrackAlt=%s\n", settings.previousTrackAlt);
-        fprintf(file, "nextTrackAlt=%s\n", settings.nextTrackAlt);
-        fprintf(file, "scrollUpAlt=%s\n", settings.scrollUpAlt);
-        fprintf(file, "scrollDownAlt=%s\n", settings.scrollDownAlt);
-        fprintf(file, "switchNumberedSong=%s\n", settings.switchNumberedSong);
-        fprintf(file, "togglePause=%s\n", settings.togglePause);
-        fprintf(file, "toggleColorsDerivedFrom=%s\n", settings.toggleColorsDerivedFrom);
-        fprintf(file, "toggleVisualizer=%s\n", settings.toggleVisualizer);
-        fprintf(file, "toggleCovers=%s\n", settings.toggleCovers);
-        fprintf(file, "toggleAscii=%s\n", settings.toggleAscii);
-        fprintf(file, "toggleRepeat=%s\n", settings.toggleRepeat);
-        fprintf(file, "toggleShuffle=%s\n", settings.toggleShuffle);
-        fprintf(file, "seekBackward=%s\n", settings.seekBackward);
-        fprintf(file, "seekForward=%s\n", settings.seekForward);
-        fprintf(file, "savePlaylist=%s\n", settings.savePlaylist);
-        fprintf(file, "addToMainPlaylist=%s\n", settings.addToMainPlaylist);
-        fprintf(file, "quit=%s\n\n", settings.quit);
+        fprintf(file, "path=%s\n", settings->path);
+        // fprintf(file, "useThemeColors=%s\n", settings->useThemeColors);
+        fprintf(file, "coverEnabled=%s\n", settings->coverEnabled);
+        fprintf(file, "coverAnsi=%s\n", settings->coverAnsi);
+        fprintf(file, "visualizerEnabled=%s\n", settings->visualizerEnabled);
+        fprintf(file, "visualizerHeight=%s\n", settings->visualizerHeight);
+        fprintf(file, "useProfileColors=%s\n", settings->useProfileColors);
+        fprintf(file, "allowNotifications=%s\n", settings->allowNotifications);
+        fprintf(file, "hideLogo=%s\n", settings->hideLogo);
+        fprintf(file, "hideHelp=%s\n", settings->hideHelp);
+        fprintf(file, "lastVolume=%s\n", settings->lastVolume);
+
+        fprintf(file, "\n# Cache: Set to 1 to use cache of the music library directory tree for faster startup times.\n");
+        fprintf(file, "cacheLibrary=%s\n", settings->cacheLibrary);
+
+        fprintf(file, "\n# Color values are 0=Black, 1=Red, 2=Green, 3=Yellow, 4=Blue, 5=Magenta, 6=Cyan, 7=White\n");
+        fprintf(file, "# These mostly affect the library view.\n\n");
+        fprintf(file, "# Logo color: \n");
+        fprintf(file, "color=%s\n", settings->color);
+        fprintf(file, "# Header color in library view: \n");
+        fprintf(file, "artistColor=%s\n", settings->artistColor);
+        fprintf(file, "# Now playing song text in library view: \n");
+        fprintf(file, "titleColor=%s\n", settings->titleColor);
+        fprintf(file, "# Color of enqueued songs in library view: \n");
+        fprintf(file, "enqueuedColor=%s\n", settings->enqueuedColor);
+
+        fprintf(file, "\n# Key Bindings:\n\n");
+        fprintf(file, "volumeUp=%s\n", settings->volumeUp);
+        fprintf(file, "volumeUpAlt=%s\n", settings->volumeUpAlt);
+        fprintf(file, "volumeDown=%s\n", settings->volumeDown);
+        fprintf(file, "previousTrackAlt=%s\n", settings->previousTrackAlt);
+        fprintf(file, "nextTrackAlt=%s\n", settings->nextTrackAlt);
+        fprintf(file, "scrollUpAlt=%s\n", settings->scrollUpAlt);
+        fprintf(file, "scrollDownAlt=%s\n", settings->scrollDownAlt);
+        fprintf(file, "switchNumberedSong=%s\n", settings->switchNumberedSong);
+        fprintf(file, "togglePause=%s\n", settings->togglePause);
+        fprintf(file, "toggleColorsDerivedFrom=%s\n", settings->toggleColorsDerivedFrom);
+        fprintf(file, "toggleVisualizer=%s\n", settings->toggleVisualizer);
+        fprintf(file, "toggleCovers=%s\n", settings->toggleCovers);
+        fprintf(file, "toggleAscii=%s\n", settings->toggleAscii);
+        fprintf(file, "toggleRepeat=%s\n", settings->toggleRepeat);
+        fprintf(file, "toggleShuffle=%s\n", settings->toggleShuffle);
+        fprintf(file, "seekBackward=%s\n", settings->seekBackward);
+        fprintf(file, "seekForward=%s\n", settings->seekForward);
+        fprintf(file, "savePlaylist=%s\n", settings->savePlaylist);
+        fprintf(file, "addToMainPlaylist=%s\n", settings->addToMainPlaylist);
+        fprintf(file, "updateLibrary=%s\n", settings->updateLibrary);
+
+        fprintf(file, "quit=%s\n\n", settings->quit);
         fprintf(file, "# For special keys use terminal codes: OS, for F4 for instance. This can depend on the terminal.\n");
         fprintf(file, "# You can find out the codes for the keys by using tools like showkey.\n");
         fprintf(file, "# For special keys, see the key value after the bracket \"[\" after typing \"showkey -a\" in the terminal and then pressing a key you want info about.\n");
@@ -462,4 +481,5 @@ void setConfig()
 
         fclose(file);
         free(filepath);
+        free(configdir);
 }
