@@ -54,6 +54,14 @@ int m4aDecoderIndex = -1;
 int opusDecoderIndex = -1;
 int vorbisDecoderIndex = -1;
 
+void logTime(const char *message)
+{
+        (void)message;
+        // struct timespec ts;
+        // clock_gettime(CLOCK_REALTIME, &ts);
+        // printf("[%ld.%09ld] %s\n", ts.tv_sec, ts.tv_nsec, message);
+}
+
 enum AudioImplementation getCurrentImplementationType()
 {
         return currentImplementation;
@@ -1098,12 +1106,14 @@ void sanitize_filepath(const char *input, char *sanitized, size_t size)
         sanitized[j] = '\0';
 }
 
-char *remove_blacklisted_chars(const char *input, const char *blacklist)
+char *removeBlacklistedChars(const char *input, const char *blacklist)
 {
         if (!input || !blacklist)
+        {
                 return NULL;
+        }
 
-        char *output = malloc(strlen(input) + 1);
+        char *output = calloc(strlen(input) + 1, sizeof(char));
         if (!output)
         {
                 perror("Failed to allocate memory");
@@ -1121,7 +1131,6 @@ char *remove_blacklisted_chars(const char *input, const char *blacklist)
                 }
                 in_ptr++;
         }
-        *out_ptr = '\0';
 
         return output;
 }
@@ -1131,48 +1140,61 @@ gint64 getLengthInSec(double duration)
         return floor(llround(duration * G_USEC_PER_SEC));
 }
 
-int displaySongNotification(const char *artist, const char *title, const char *cover) {
-    if (!allowNotifications)
-        return 0;
-
-    char sanitized_cover[MAXPATHLEN];
-
-    const char *blacklist = "&;`|*~<>^()[]{}$\\\"";
-    char *sanitizedArtist = remove_blacklisted_chars(artist, blacklist);
-    char *sanitizedTitle = remove_blacklisted_chars(title, blacklist);
-
-    sanitize_filepath(cover, sanitized_cover, sizeof(sanitized_cover));
-
-    char message[MAXPATHLEN + 1024];
-    if (strlen(artist) > 0) {
-        snprintf(message, sizeof(message), "%s - %s", sanitizedArtist, sanitizedTitle);
-    } else {
-        snprintf(message, sizeof(message), "%s", sanitizedTitle);
-    }
-
-    char *args[] = {"/usr/bin/notify-send", "-a", "kew", message, "--icon", sanitized_cover, NULL};
-
-    pid_t pid = fork();
-    if (pid == -1) {
-        // Handle error
-        perror("fork");
-    } else if (pid > 0) {
-        // Parent process
-        int status;
-        waitpid(pid, &status, 0);
-    } else {
-        // Child process
-        extern char **environ;
-        if (execve("/usr/bin/notify-send", args, environ) == -1) {
-
-            _Exit(EXIT_FAILURE);
+int displaySongNotification(const char *artist, const char *title, const char *cover)
+{
+        if (!allowNotifications)
+        {
+                return 0;
         }
-    }
 
-    free(sanitizedArtist);
-    free(sanitizedTitle);
+        char sanitized_cover[MAXPATHLEN];
+        const char *blacklist = "&;`|*~<>^()[]{}$\\\"";
+        char *sanitizedArtist = removeBlacklistedChars(artist, blacklist);
+        char *sanitizedTitle = removeBlacklistedChars(title, blacklist);
 
-    return 0;
+        if (!sanitizedArtist || !sanitizedTitle)
+        {
+                free(sanitizedArtist);
+                free(sanitizedTitle);
+                return -1;
+        }
+
+        sanitize_filepath(cover, sanitized_cover, sizeof(sanitized_cover));
+
+        char message[MAXPATHLEN + 1024];
+        if (strlen(artist) > 0)
+        {
+                snprintf(message, sizeof(message), "%s - %s", sanitizedArtist, sanitizedTitle);
+        }
+        else
+        {
+                snprintf(message, sizeof(message), "%s", sanitizedTitle);
+        }
+
+        char *args[] = {"/usr/bin/notify-send", "-a", "kew", message, "--icon", sanitized_cover, NULL};
+
+        pid_t pid = vfork();
+        if (pid == -1)
+        {
+                perror("vfork");
+                free(sanitizedArtist);
+                free(sanitizedTitle);
+                return -1;
+        }
+        else if (pid == 0)
+        {
+                // Child process
+                execv(args[0], args);
+                // If execv fails
+                perror("execv");
+                _Exit(EXIT_FAILURE);
+        }
+
+        // Parent process
+        free(sanitizedArtist);
+        free(sanitizedTitle);
+
+        return 0;
 }
 
 void executeSwitch(AudioData *pAudioData)
